@@ -42,6 +42,11 @@
 #
 # (*)	presented for reference/comparison purposes;
 
+
+# The first two arguments should always be the flavour and output file path.
+if ($#ARGV < 1) { die "Not enough arguments provided.
+  Two arguments are necessary: the flavour and the output file path."; }
+
 $flavour = shift;
 $output  = shift;
 
