@@ -87,7 +87,7 @@ ASN1_SEQUENCE_enc(X509_CINF, enc, 0) = {
 } ASN1_SEQUENCE_END_enc(X509_CINF, X509_CINF)
 
 IMPLEMENT_ASN1_FUNCTIONS(X509_CINF)
-/* X509 top level structure needs a bit of customisation */
+// X509 top level structure needs a bit of customisation
 
 extern void policy_cache_free(X509_POLICY_CACHE *cache);
 
@@ -114,27 +114,27 @@ static int x509_cb(int operation, ASN1_VALUE **pval, const ASN1_ITEM *it,
       break;
 
     case ASN1_OP_D2I_POST: {
-      /* The version must be one of v1(0), v2(1), or v3(2). */
+      // The version must be one of v1(0), v2(1), or v3(2).
       long version = X509_VERSION_1;
       if (ret->cert_info->version != NULL) {
         version = ASN1_INTEGER_get(ret->cert_info->version);
-        /* TODO(https://crbug.com/boringssl/364): |X509_VERSION_1| should
-         * also be rejected here. This means an explicitly-encoded X.509v1
-         * version. v1 is DEFAULT, so DER requires it be omitted. */
+        // TODO(https://crbug.com/boringssl/364): |X509_VERSION_1| should
+        // also be rejected here. This means an explicitly-encoded X.509v1
+        // version. v1 is DEFAULT, so DER requires it be omitted.
         if (version < X509_VERSION_1 || version > X509_VERSION_3) {
           OPENSSL_PUT_ERROR(X509, X509_R_INVALID_VERSION);
           return 0;
         }
       }
 
-      /* Per RFC 5280, section 4.1.2.8, these fields require v2 or v3. */
+      // Per RFC 5280, section 4.1.2.8, these fields require v2 or v3.
       if (version == X509_VERSION_1 && (ret->cert_info->issuerUID != NULL ||
                                         ret->cert_info->subjectUID != NULL)) {
         OPENSSL_PUT_ERROR(X509, X509_R_INVALID_FIELD_FOR_VERSION);
         return 0;
       }
 
-      /* Per RFC 5280, section 4.1.2.9, extensions require v3. */
+      // Per RFC 5280, section 4.1.2.9, extensions require v3.
       if (version != X509_VERSION_3 && ret->cert_info->extensions != NULL) {
         OPENSSL_PUT_ERROR(X509, X509_R_INVALID_FIELD_FOR_VERSION);
         return 0;
@@ -224,12 +224,10 @@ void *X509_get_ex_data(X509 *r, int idx) {
   return (CRYPTO_get_ex_data(&r->ex_data, idx));
 }
 
-/*
- * X509_AUX ASN1 routines. X509_AUX is the name given to a certificate with
- * extra info tagged on the end. Since these functions set how a certificate
- * is trusted they should only be used when the certificate comes from a
- * reliable source such as local storage.
- */
+// X509_AUX ASN1 routines. X509_AUX is the name given to a certificate with
+// extra info tagged on the end. Since these functions set how a certificate
+// is trusted they should only be used when the certificate comes from a
+// reliable source such as local storage.
 
 X509 *d2i_X509_AUX(X509 **a, const unsigned char **pp, long length) {
   const unsigned char *q = *pp;
@@ -240,13 +238,13 @@ X509 *d2i_X509_AUX(X509 **a, const unsigned char **pp, long length) {
     freeret = 1;
   }
   ret = d2i_X509(a, &q, length);
-  /* If certificate unreadable then forget it */
+  // If certificate unreadable then forget it
   if (!ret) {
     return NULL;
   }
-  /* update length */
+  // update length
   length -= q - *pp;
-  /* Parse auxiliary information if there is any. */
+  // Parse auxiliary information if there is any.
   if (length > 0 && !d2i_X509_CERT_AUX(&ret->aux, &q, length)) {
     goto err;
   }
@@ -262,23 +260,19 @@ err:
   return NULL;
 }
 
-/*
- * Serialize trusted certificate to *pp or just return the required buffer
- * length if pp == NULL.  We ultimately want to avoid modifying *pp in the
- * error path, but that depends on similar hygiene in lower-level functions.
- * Here we avoid compounding the problem.
- */
+// Serialize trusted certificate to *pp or just return the required buffer
+// length if pp == NULL.  We ultimately want to avoid modifying *pp in the
+// error path, but that depends on similar hygiene in lower-level functions.
+// Here we avoid compounding the problem.
 static int i2d_x509_aux_internal(X509 *a, unsigned char **pp) {
   int length, tmplen;
   unsigned char *start = pp != NULL ? *pp : NULL;
 
   assert(pp == NULL || *pp != NULL);
 
-  /*
-   * This might perturb *pp on error, but fixing that belongs in i2d_X509()
-   * not here.  It should be that if a == NULL length is zero, but we check
-   * both just in case.
-   */
+  // This might perturb *pp on error, but fixing that belongs in i2d_X509()
+  // not here.  It should be that if a == NULL length is zero, but we check
+  // both just in case.
   length = i2d_X509(a, pp);
   if (length <= 0 || a == NULL) {
     return length;
@@ -298,36 +292,34 @@ static int i2d_x509_aux_internal(X509 *a, unsigned char **pp) {
   return length;
 }
 
-/*
- * Serialize trusted certificate to *pp, or just return the required buffer
- * length if pp == NULL.
- *
- * When pp is not NULL, but *pp == NULL, we allocate the buffer, but since
- * we're writing two ASN.1 objects back to back, we can't have i2d_X509() do
- * the allocation, nor can we allow i2d_X509_CERT_AUX() to increment the
- * allocated buffer.
- */
+// Serialize trusted certificate to *pp, or just return the required buffer
+// length if pp == NULL.
+//
+// When pp is not NULL, but *pp == NULL, we allocate the buffer, but since
+// we're writing two ASN.1 objects back to back, we can't have i2d_X509() do
+// the allocation, nor can we allow i2d_X509_CERT_AUX() to increment the
+// allocated buffer.
 int i2d_X509_AUX(X509 *a, unsigned char **pp) {
   int length;
   unsigned char *tmp;
 
-  /* Buffer provided by caller */
+  // Buffer provided by caller
   if (pp == NULL || *pp != NULL) {
     return i2d_x509_aux_internal(a, pp);
   }
 
-  /* Obtain the combined length */
+  // Obtain the combined length
   if ((length = i2d_x509_aux_internal(a, NULL)) <= 0) {
     return length;
   }
 
-  /* Allocate requisite combined storage */
+  // Allocate requisite combined storage
   *pp = tmp = OPENSSL_malloc(length);
   if (tmp == NULL) {
-    return -1; /* Push error onto error stack? */
+    return -1;  // Push error onto error stack?
   }
 
-  /* Encode, but keep *pp at the originally malloced pointer */
+  // Encode, but keep *pp at the originally malloced pointer
   length = i2d_x509_aux_internal(a, &tmp);
   if (length <= 0) {
     OPENSSL_free(*pp);
@@ -346,11 +338,8 @@ int i2d_X509_tbs(X509 *x509, unsigned char **outp) {
 }
 
 int X509_set1_signature_algo(X509 *x509, const X509_ALGOR *algo) {
-  /* TODO(https://crbug.com/boringssl/407): Generated ASN.1 dup functions
-   * should be const. Alternatively, when we can embed required fields
-   * directly in structs, import |X509_ALGOR_copy| from upstream. */
-  X509_ALGOR *copy1 = X509_ALGOR_dup((X509_ALGOR *)algo);
-  X509_ALGOR *copy2 = X509_ALGOR_dup((X509_ALGOR *)algo);
+  X509_ALGOR *copy1 = X509_ALGOR_dup(algo);
+  X509_ALGOR *copy2 = X509_ALGOR_dup(algo);
   if (copy1 == NULL || copy2 == NULL) {
     X509_ALGOR_free(copy1);
     X509_ALGOR_free(copy2);
