@@ -13,35 +13,41 @@
  * permissions and limitations under the License.
  */
 
+#include <fcntl.h>
+#include <getopt.h>
+#include <inttypes.h>
+#include <netdb.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdlib.h>
 #include <sys/param.h>
 #include <sys/socket.h>
-#include <netdb.h>
-
 #include <unistd.h>
-#include <getopt.h>
-#include <fcntl.h>
 
 #ifndef S2N_INTERN_LIBCRYPTO
-#include <openssl/crypto.h>
-#include <openssl/err.h>
+    #include <openssl/crypto.h>
+    #include <openssl/err.h>
 #endif
 
 #include "api/s2n.h"
-#include "api/unstable/renegotiate.h"
 #include "api/unstable/npn.h"
+#include "api/unstable/renegotiate.h"
 #include "common.h"
 #include "error/s2n_errno.h"
-
 #include "tls/s2n_connection.h"
 
-#define OPT_TICKET_IN 1000
-#define OPT_TICKET_OUT 1001
-#define OPT_SEND_FILE 1002
-#define OPT_RENEG 1003
-#define OPT_NPN 1004
+#define OPT_TICKET_IN          1000
+#define OPT_TICKET_OUT         1001
+#define OPT_SEND_FILE          1002
+#define OPT_RENEG              1003
+#define OPT_NPN                1004
+#define OPT_PREFER_LOW_LATENCY 1005
+#define OPT_PREFER_THROUGHPUT  1006
+#define OPT_BUFFERED_SEND      1007
 
 void usage()
 {
+    /* clang-format off */
     fprintf(stderr, "usage: s2nc [options] host [port]\n");
     fprintf(stderr, " host: hostname or IP address to connect to\n");
     fprintf(stderr, " port: port to connect to\n");
@@ -107,8 +113,15 @@ void usage()
                     "    reject: Reject all server requests for a new handshake\n"
                     "    wait: Wait for additional application data before accepting server requests. Intended for the integ tests.\n");
     fprintf(stderr, "  --npn \n");
-    fprintf(stderr, "    Indicates support for the NPN extension. The '--alpn' option MUST be used with this option to signal the protocols supported."); 
+    fprintf(stderr, "    Indicates support for the NPN extension. The '--alpn' option MUST be used with this option to signal the protocols supported.");
     fprintf(stderr, "\n");
+    fprintf(stderr, "  --buffered-send <buffer size>\n");
+    fprintf(stderr, "    Set s2n_send to buffer up to <buffer size> bytes before sending records over the wire.\n");
+    fprintf(stderr, "  --prefer-low-latency\n");
+    fprintf(stderr, "    Prefer low latency by clamping maximum outgoing record size at 1500.\n");
+    fprintf(stderr, "  --prefer-throughput\n");
+    fprintf(stderr, "    Prefer throughput by raising maximum outgoing record size to 16k\n");
+    /* clang-format on */
     exit(1);
 }
 
@@ -121,13 +134,13 @@ static int test_session_ticket_cb(struct s2n_connection *conn, void *ctx, struct
 
     GUARD_EXIT(s2n_session_ticket_get_data_len(ticket, &session_state_length), "Error getting ticket length ");
     session_state = realloc(session_state, session_state_length);
-    if(session_state == NULL) {
+    if (session_state == NULL) {
         print_s2n_error("Error getting new session state");
         exit(1);
     }
     GUARD_EXIT(s2n_session_ticket_get_data(ticket, session_state_length, session_state), "Error getting ticket data");
 
-    bool *session_ticket_recv = (bool *)ctx;
+    bool *session_ticket_recv = (bool *) ctx;
     *session_ticket_recv = 1;
 
     return S2N_SUCCESS;
@@ -154,8 +167,8 @@ static int reneg_req_cb(struct s2n_connection *conn, void *context, s2n_renegoti
 }
 
 static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs, s2n_status_request_type type,
-    struct verify_data *unsafe_verify_data, const char *host, const char *alpn_protocols, uint16_t mfl_value) {
-
+        struct verify_data *unsafe_verify_data, const char *host, const char *alpn_protocols, uint16_t mfl_value)
+{
     if (config == NULL) {
         print_s2n_error("Error getting new config");
         exit(1);
@@ -170,7 +183,7 @@ static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs
     }
 
     if (type == S2N_STATUS_REQUEST_OCSP) {
-        if(s2n_config_set_check_stapled_ocsp_response(config, 1)) {
+        if (s2n_config_set_check_stapled_ocsp_response(config, 1)) {
             print_s2n_error("OCSP validation is not supported by the linked libCrypto implementation. It cannot be set.");
         }
     }
@@ -227,7 +240,7 @@ static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs
             protocols[idx][length] = '\0';
         }
 
-        GUARD_EXIT(s2n_config_set_protocol_preferences(config, (const char *const *)protocols, protocol_count), "Failed to set protocol preferences");
+        GUARD_EXIT(s2n_config_set_protocol_preferences(config, (const char *const *) protocols, protocol_count), "Failed to set protocol preferences");
 
         while (protocol_count) {
             protocol_count--;
@@ -238,7 +251,7 @@ static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs
 
     uint8_t mfl_code = 0;
     if (mfl_value > 0) {
-        switch(mfl_value) {
+        switch (mfl_value) {
             case 512:
                 mfl_code = S2N_TLS_MAX_FRAG_LEN_512;
                 break;
@@ -301,36 +314,42 @@ int main(int argc, char *const *argv)
     bool setup_reneg_cb = false;
     struct reneg_req_ctx reneg_ctx = { 0 };
     bool npn = false;
+    uint32_t send_buffer_size = 0;
+    bool prefer_low_latency = false;
+    bool prefer_throughput = false;
 
     static struct option long_options[] = {
-        {"alpn", required_argument, 0, 'a'},
-        {"ciphers", required_argument, 0, 'c'},
-        {"enter-fips-mode", no_argument, NULL, 'F'},
-        {"echo", no_argument, 0, 'e'},
-        {"send-file", required_argument, 0, OPT_SEND_FILE},
-        {"help", no_argument, 0, 'h'},
-        {"name", required_argument, 0, 'n'},
-        {"status", no_argument, 0, 's'},
-        {"mfl", required_argument, 0, 'm'},
-        {"ca-file", required_argument, 0, 'f'},
-        {"ca-dir", required_argument, 0, 'd'},
-        {"cert", required_argument, 0, 'l'},
-        {"key", required_argument, 0, 'k'},
-        {"insecure", no_argument, 0, 'i'},
-        {"reconnect", no_argument, 0, 'r'},
-        {"ticket-out", required_argument, 0, OPT_TICKET_OUT},
-        {"ticket-in", required_argument, 0, OPT_TICKET_IN},
-        {"no-session-ticket", no_argument, 0, 'T'},
-        {"dynamic", required_argument, 0, 'D'},
-        {"timeout", required_argument, 0, 't'},
-        {"corked-io", no_argument, 0, 'C'},
-        {"tls13", no_argument, 0, '3'},
-        {"non-blocking", no_argument, 0, 'B'},
-        {"key-log", required_argument, 0, 'L'},
-        {"psk", required_argument, 0, 'P'},
-        {"early-data", required_argument, 0, 'E'},
-        {"renegotiation", required_argument, 0, OPT_RENEG},
-        {"npn", no_argument, 0, OPT_NPN},
+        { "alpn", required_argument, 0, 'a' },
+        { "ciphers", required_argument, 0, 'c' },
+        { "enter-fips-mode", no_argument, NULL, 'F' },
+        { "echo", no_argument, 0, 'e' },
+        { "send-file", required_argument, 0, OPT_SEND_FILE },
+        { "help", no_argument, 0, 'h' },
+        { "name", required_argument, 0, 'n' },
+        { "status", no_argument, 0, 's' },
+        { "mfl", required_argument, 0, 'm' },
+        { "ca-file", required_argument, 0, 'f' },
+        { "ca-dir", required_argument, 0, 'd' },
+        { "cert", required_argument, 0, 'l' },
+        { "key", required_argument, 0, 'k' },
+        { "insecure", no_argument, 0, 'i' },
+        { "reconnect", no_argument, 0, 'r' },
+        { "ticket-out", required_argument, 0, OPT_TICKET_OUT },
+        { "ticket-in", required_argument, 0, OPT_TICKET_IN },
+        { "no-session-ticket", no_argument, 0, 'T' },
+        { "dynamic", required_argument, 0, 'D' },
+        { "timeout", required_argument, 0, 't' },
+        { "corked-io", no_argument, 0, 'C' },
+        { "tls13", no_argument, 0, '3' },
+        { "non-blocking", no_argument, 0, 'B' },
+        { "key-log", required_argument, 0, 'L' },
+        { "psk", required_argument, 0, 'P' },
+        { "early-data", required_argument, 0, 'E' },
+        { "renegotiation", required_argument, 0, OPT_RENEG },
+        { "npn", no_argument, 0, OPT_NPN },
+        { "buffered-send", required_argument, 0, OPT_BUFFERED_SEND },
+        { "prefer-low-latency", no_argument, NULL, OPT_PREFER_LOW_LATENCY },
+        { "prefer-throughput", no_argument, NULL, OPT_PREFER_THROUGHPUT },
         { 0 },
     };
 
@@ -341,116 +360,131 @@ int main(int argc, char *const *argv)
             break;
         }
         switch (c) {
-        case 'a':
-            alpn_protocols = optarg;
-            break;
-        case 'C':
-            use_corked_io = 1;
-            break;
-        case 'c':
-            cipher_prefs = optarg;
-            break;
-        case 'F':
-            fips_mode = 1;
-            break;
-        case 'e':
-            echo_input = true;
-            break;
-        case OPT_SEND_FILE:
-            send_file = load_file_to_cstring(optarg);
-            break;
-        case 'h':
-            usage();
-            break;
-        case 'n':
-            server_name = optarg;
-            break;
-        case 's':
-            type = S2N_STATUS_REQUEST_OCSP;
-            break;
-        case 'm':
-            mfl_value = (uint16_t) atoi(optarg);
-            break;
-        case 'f':
-            ca_file = optarg;
-            break;
-        case 'd':
-            ca_dir = optarg;
-            break;
-        case 'l':
-            client_cert = load_file_to_cstring(optarg);
-            client_cert_input = true;
-            break;
-        case 'k':
-            client_key = load_file_to_cstring(optarg);
-            client_key_input = true;
-            break;
-        case 'i':
-            insecure = 1;
-            break;
-        case 'r':
-            reconnect = 5;
-            break;
-        case OPT_TICKET_OUT:
-            ticket_out = optarg;
-            break;
-        case OPT_TICKET_IN:
-            ticket_in = optarg;
-            break;
-        case 'T':
-            session_ticket = 0;
-            break;
-        case 't':
-            dyn_rec_timeout = (uint8_t) MIN(255, atoi(optarg));
-            break;
-        case 'D':
-            errno = 0;
-            dyn_rec_threshold = strtoul(optarg, 0, 10);
-            if (errno == ERANGE) {
-                dyn_rec_threshold = 0;
+            case 'a':
+                alpn_protocols = optarg;
+                break;
+            case 'C':
+                use_corked_io = 1;
+                break;
+            case 'c':
+                cipher_prefs = optarg;
+                break;
+            case 'F':
+                fips_mode = 1;
+                break;
+            case 'e':
+                echo_input = true;
+                break;
+            case OPT_SEND_FILE:
+                send_file = load_file_to_cstring(optarg);
+                break;
+            case 'h':
+                usage();
+                break;
+            case 'n':
+                server_name = optarg;
+                break;
+            case 's':
+                type = S2N_STATUS_REQUEST_OCSP;
+                break;
+            case 'm':
+                mfl_value = (uint16_t) atoi(optarg);
+                break;
+            case 'f':
+                ca_file = optarg;
+                break;
+            case 'd':
+                ca_dir = optarg;
+                break;
+            case 'l':
+                client_cert = load_file_to_cstring(optarg);
+                client_cert_input = true;
+                break;
+            case 'k':
+                client_key = load_file_to_cstring(optarg);
+                client_key_input = true;
+                break;
+            case 'i':
+                insecure = 1;
+                break;
+            case 'r':
+                reconnect = 5;
+                break;
+            case OPT_TICKET_OUT:
+                ticket_out = optarg;
+                break;
+            case OPT_TICKET_IN:
+                ticket_in = optarg;
+                break;
+            case 'T':
+                session_ticket = 0;
+                break;
+            case 't':
+                dyn_rec_timeout = (uint8_t) MIN(255, atoi(optarg));
+                break;
+            case 'D':
+                errno = 0;
+                dyn_rec_threshold = strtoul(optarg, 0, 10);
+                if (errno == ERANGE) {
+                    dyn_rec_threshold = 0;
+                }
+                break;
+            case '3':
+                /* Do nothing -- this argument is deprecated. */
+                break;
+            case 'B':
+                non_blocking = 1;
+                break;
+            case 'L':
+                key_log_path = optarg;
+                break;
+            case 'P':
+                if (psk_list_len >= S2N_MAX_PSK_LIST_LENGTH) {
+                    fprintf(stderr, "Error setting psks, maximum number of psks permitted is 10.\n");
+                    exit(1);
+                }
+                psk_optarg_list[psk_list_len++] = optarg;
+                break;
+            case 'E':
+                early_data = load_file_to_cstring(optarg);
+                GUARD_EXIT_NULL(early_data);
+                break;
+            case OPT_RENEG:
+                setup_reneg_cb = true;
+                if (strcmp(optarg, "accept") == 0) {
+                    reneg_ctx.response = S2N_RENEGOTIATE_ACCEPT;
+                } else if (strcmp(optarg, "reject") == 0) {
+                    reneg_ctx.response = S2N_RENEGOTIATE_REJECT;
+                } else if (strcmp(optarg, "wait") == 0) {
+                    reneg_ctx.response = S2N_RENEGOTIATE_ACCEPT;
+                    reneg_ctx.wait = true;
+                } else {
+                    fprintf(stderr, "Unrecognized option: %s\n", optarg);
+                    exit(1);
+                }
+                break;
+            case OPT_NPN:
+                npn = true;
+                break;
+            case OPT_BUFFERED_SEND: {
+                intmax_t send_buffer_size_scanned_value = strtoimax(optarg, 0, 10);
+                if (send_buffer_size_scanned_value > UINT32_MAX || send_buffer_size_scanned_value < 0) {
+                    fprintf(stderr, "<buffer size> must be a positive 32 bit value\n");
+                    exit(1);
+                }
+                send_buffer_size = (uint32_t) send_buffer_size_scanned_value;
+                break;
             }
-            break;
-        case '3':
-            /* Do nothing -- this argument is deprecated. */
-            break;
-        case 'B':
-            non_blocking = 1;
-            break;
-        case 'L':
-            key_log_path = optarg;
-            break;
-        case 'P':
-            if (psk_list_len >= S2N_MAX_PSK_LIST_LENGTH) {
-                fprintf(stderr, "Error setting psks, maximum number of psks permitted is 10.\n");
-                exit(1);
-            }
-            psk_optarg_list[psk_list_len++] = optarg;
-            break;
-        case 'E':
-            early_data = load_file_to_cstring(optarg);
-            GUARD_EXIT_NULL(early_data);
-            break;
-        case OPT_RENEG:
-            setup_reneg_cb = true;
-            if (strcmp(optarg, "accept") == 0) {
-                reneg_ctx.response = S2N_RENEGOTIATE_ACCEPT;
-            } else if (strcmp(optarg, "reject") == 0) {
-                reneg_ctx.response = S2N_RENEGOTIATE_REJECT;
-            } else if (strcmp(optarg, "wait") == 0) {
-                reneg_ctx.response = S2N_RENEGOTIATE_ACCEPT;
-                reneg_ctx.wait = true;
-            } else {
-                fprintf(stderr, "Unrecognized option: %s\n", optarg);
-                exit(1);
-            }
-            break;
-        case OPT_NPN:
-            npn = true;
-            break;
-        case '?':
-        default:
-            usage();
-            break;
+            case OPT_PREFER_LOW_LATENCY:
+                prefer_low_latency = true;
+                break;
+            case OPT_PREFER_THROUGHPUT:
+                prefer_throughput = true;
+                break;
+            case '?':
+            default:
+                usage();
+                break;
         }
     }
 
@@ -483,7 +517,7 @@ int main(int argc, char *const *argv)
 
     if (fips_mode) {
 #ifndef S2N_INTERN_LIBCRYPTO
-#if defined(OPENSSL_FIPS) || defined(OPENSSL_IS_AWSLC)
+    #if defined(OPENSSL_FIPS) || defined(OPENSSL_IS_AWSLC)
         if (FIPS_mode_set(1) == 0) {
             unsigned long fips_rc = ERR_get_error();
             char ssl_error_buf[256]; /* Openssl claims you need no more than 120 bytes for error strings */
@@ -491,11 +525,16 @@ int main(int argc, char *const *argv)
             exit(1);
         }
         printf("s2nc entered FIPS mode\n");
-#else
+    #else
         fprintf(stderr, "Error entering FIPS mode. s2nc was not built against a FIPS-capable libcrypto.\n");
         exit(1);
+    #endif
 #endif
-#endif
+    }
+
+    if (prefer_low_latency && prefer_throughput) {
+        fprintf(stderr, "prefer-throughput and prefer-low-latency options are mutually exclusive\n");
+        exit(1);
     }
 
     GUARD_EXIT(s2n_init(), "Error running s2n_init()");
@@ -538,6 +577,10 @@ int main(int argc, char *const *argv)
         struct s2n_config *config = s2n_config_new();
         setup_s2n_config(config, cipher_prefs, type, &unsafe_verify_data, host, alpn_protocols, mfl_value);
 
+        if (send_buffer_size != 0) {
+            GUARD_EXIT(s2n_config_set_send_buffer_size(config, send_buffer_size), "Error setting send buffer size");
+        }
+
         if (client_cert_input != client_key_input) {
             print_s2n_error("Client cert/key pair must be given.");
         }
@@ -553,8 +596,7 @@ int main(int argc, char *const *argv)
             if (s2n_config_set_verification_ca_location(config, ca_file, ca_dir) < 0) {
                 print_s2n_error("Error setting CA file for trust store.");
             }
-        }
-        else if (insecure) {
+        } else if (insecure) {
             GUARD_EXIT(s2n_config_disable_x509_verification(config), "Error disabling X.509 validation");
         }
 
@@ -568,13 +610,11 @@ int main(int argc, char *const *argv)
             key_log_file = fopen(key_log_path, "a");
             GUARD_EXIT(key_log_file == NULL ? S2N_FAILURE : S2N_SUCCESS, "Failed to open key log file");
             GUARD_EXIT(
-                s2n_config_set_key_log_cb(
-                    config,
-                    key_log_callback,
-                    (void *)key_log_file
-                ),
-                "Failed to set key log callback"
-            );
+                    s2n_config_set_key_log_cb(
+                            config,
+                            key_log_callback,
+                            (void *) key_log_file),
+                    "Failed to set key log callback");
         }
 
         if (setup_reneg_cb) {
@@ -594,10 +634,10 @@ int main(int argc, char *const *argv)
         }
 
         GUARD_EXIT(s2n_connection_set_config(conn, config), "Error setting configuration");
- 
+
         GUARD_EXIT(s2n_set_server_name(conn, server_name), "Error setting server name");
 
-        GUARD_EXIT(s2n_connection_set_fd(conn, sockfd) , "Error setting file descriptor");
+        GUARD_EXIT(s2n_connection_set_fd(conn, sockfd), "Error setting file descriptor");
 
         GUARD_EXIT(s2n_connection_set_client_auth_type(conn, S2N_CERT_AUTH_OPTIONAL), "Error setting ClientAuth optional");
 
@@ -621,6 +661,14 @@ int main(int argc, char *const *argv)
 
         GUARD_EXIT(s2n_setup_external_psk_list(conn, psk_optarg_list, psk_list_len), "Error setting external psk list");
 
+        if (prefer_throughput) {
+            GUARD_RETURN(s2n_connection_prefer_throughput(conn), "Error setting prefer throughput");
+        }
+
+        if (prefer_low_latency) {
+            GUARD_RETURN(s2n_connection_prefer_low_latency(conn), "Error setting prefer low latency");
+        }
+
         if (early_data) {
             if (!session_ticket) {
                 print_s2n_error("Early data can only be used with session tickets.");
@@ -629,7 +677,7 @@ int main(int argc, char *const *argv)
             /* Send early data if we have a received a session ticket from the server */
             if (session_state_length) {
                 uint32_t early_data_length = strlen(early_data);
-                GUARD_EXIT(early_data_send(conn, (uint8_t *)early_data, early_data_length), "Error sending early data");
+                GUARD_EXIT(early_data_send(conn, (uint8_t *) early_data, early_data_length), "Error sending early data");
             }
         }
 
@@ -663,7 +711,7 @@ int main(int argc, char *const *argv)
                     exit(1);
                 }
             }
-            if(ticket_out) {
+            if (ticket_out) {
                 GUARD_EXIT(write_array_to_file(ticket_out, session_state, session_state_length), "Failed to write to ticket-out file");
             }
         }
@@ -695,16 +743,7 @@ int main(int argc, char *const *argv)
             GUARD_EXIT(renegotiate(conn, sockfd, reneg_ctx.wait), "Renegotiation failed");
         }
 
-        /* The following call can block on receiving a close_notify if we initiate the shutdown or if the */
-        /* peer fails to send a close_notify. */
-        /* TODO: However, we should expect to receive a close_notify from the peer and shutdown gracefully. */
-        /* Please see tracking issue for more detail: https://github.com/aws/s2n-tls/issues/2692 */
-        s2n_blocked_status blocked;
-        int shutdown_rc = s2n_shutdown(conn, &blocked);
-        if (shutdown_rc == -1 && blocked != S2N_BLOCKED_ON_READ) {
-            fprintf(stderr, "Unexpected error during shutdown: '%s'\n", s2n_strerror(s2n_errno, "NULL"));
-            exit(1);
-        }
+        GUARD_EXIT(wait_for_shutdown(conn, sockfd), "Error closing connection");
 
         GUARD_EXIT(s2n_connection_free(conn), "Error freeing connection");
 
