@@ -13,53 +13,57 @@
  * permissions and limitations under the License.
  */
 
+#include <arpa/inet.h>
+#include <openssl/asn1.h>
+#include <openssl/err.h>
+#include <openssl/x509.h>
+#include <sys/socket.h>
+
 #include "crypto/s2n_openssl.h"
 #include "crypto/s2n_openssl_x509.h"
-#include "utils/s2n_asn1_time.h"
-#include "utils/s2n_result.h"
-#include "utils/s2n_safety.h"
-#include "utils/s2n_rfc5952.h"
 #include "tls/extensions/s2n_extension_list.h"
 #include "tls/s2n_config.h"
 #include "tls/s2n_connection.h"
 #include "tls/s2n_crl.h"
-
-#include <arpa/inet.h>
-#include <sys/socket.h>
-
-#include <openssl/err.h>
-#include <openssl/asn1.h>
-#include <openssl/x509.h>
+#include "utils/s2n_asn1_time.h"
+#include "utils/s2n_result.h"
+#include "utils/s2n_rfc5952.h"
+#include "utils/s2n_safety.h"
 
 #if S2N_OCSP_STAPLING_SUPPORTED
-#include <openssl/ocsp.h>
-DEFINE_POINTER_CLEANUP_FUNC(OCSP_RESPONSE*, OCSP_RESPONSE_free);
-DEFINE_POINTER_CLEANUP_FUNC(OCSP_BASICRESP*, OCSP_BASICRESP_free);
+    #include <openssl/ocsp.h>
+DEFINE_POINTER_CLEANUP_FUNC(OCSP_RESPONSE *, OCSP_RESPONSE_free);
+DEFINE_POINTER_CLEANUP_FUNC(OCSP_BASICRESP *, OCSP_BASICRESP_free);
 #endif
 
 #ifndef X509_V_FLAG_PARTIAL_CHAIN
-#define X509_V_FLAG_PARTIAL_CHAIN 0x80000
+    #define X509_V_FLAG_PARTIAL_CHAIN 0x80000
 #endif
 
 #define DEFAULT_MAX_CHAIN_DEPTH 7
 /* Time used by default for nextUpdate if none provided in OCSP: 1 hour since thisUpdate. */
 #define DEFAULT_OCSP_NEXT_UPDATE_PERIOD 3600000000000
 
-DEFINE_POINTER_CLEANUP_FUNC(STACK_OF(X509_CRL)*, sk_X509_CRL_free);
+DEFINE_POINTER_CLEANUP_FUNC(STACK_OF(X509_CRL) *, sk_X509_CRL_free);
+DEFINE_POINTER_CLEANUP_FUNC(STACK_OF(GENERAL_NAME) *, GENERAL_NAMES_free);
 
-uint8_t s2n_x509_ocsp_stapling_supported(void) {
+uint8_t s2n_x509_ocsp_stapling_supported(void)
+{
     return S2N_OCSP_STAPLING_SUPPORTED;
 }
 
-void s2n_x509_trust_store_init_empty(struct s2n_x509_trust_store *store) {
+void s2n_x509_trust_store_init_empty(struct s2n_x509_trust_store *store)
+{
     store->trust_store = NULL;
 }
 
-uint8_t s2n_x509_trust_store_has_certs(struct s2n_x509_trust_store *store) {
+uint8_t s2n_x509_trust_store_has_certs(struct s2n_x509_trust_store *store)
+{
     return store->trust_store ? (uint8_t) 1 : (uint8_t) 0;
 }
 
-int s2n_x509_trust_store_from_system_defaults(struct s2n_x509_trust_store *store) {
+int s2n_x509_trust_store_from_system_defaults(struct s2n_x509_trust_store *store)
+{
     if (!store->trust_store) {
         store->trust_store = X509_STORE_new();
         POSIX_ENSURE_REF(store->trust_store);
@@ -71,8 +75,6 @@ int s2n_x509_trust_store_from_system_defaults(struct s2n_x509_trust_store *store
         POSIX_BAIL(S2N_ERR_X509_TRUST_STORE);
     }
 
-    X509_STORE_set_flags(store->trust_store, X509_VP_FLAG_DEFAULT);
-
     return 0;
 }
 
@@ -85,14 +87,14 @@ int s2n_x509_trust_store_add_pem(struct s2n_x509_trust_store *store, const char
         store->trust_store = X509_STORE_new();
     }
 
-    DEFER_CLEANUP(struct s2n_stuffer pem_in_stuffer = {0}, s2n_stuffer_free);
-    DEFER_CLEANUP(struct s2n_stuffer der_out_stuffer = {0}, s2n_stuffer_free);
+    DEFER_CLEANUP(struct s2n_stuffer pem_in_stuffer = { 0 }, s2n_stuffer_free);
+    DEFER_CLEANUP(struct s2n_stuffer der_out_stuffer = { 0 }, s2n_stuffer_free);
 
     POSIX_GUARD(s2n_stuffer_alloc_ro_from_string(&pem_in_stuffer, pem));
     POSIX_GUARD(s2n_stuffer_growable_alloc(&der_out_stuffer, 2048));
 
     do {
-        DEFER_CLEANUP(struct s2n_blob next_cert = {0}, s2n_free);
+        DEFER_CLEANUP(struct s2n_blob next_cert = { 0 }, s2n_free);
 
         POSIX_GUARD(s2n_stuffer_certificate_from_pem(&pem_in_stuffer, &der_out_stuffer));
         POSIX_GUARD(s2n_alloc(&next_cert, s2n_stuffer_data_available(&der_out_stuffer)));
@@ -111,7 +113,8 @@ int s2n_x509_trust_store_add_pem(struct s2n_x509_trust_store *store, const char
     return 0;
 }
 
-int s2n_x509_trust_store_from_ca_file(struct s2n_x509_trust_store *store, const char *ca_pem_filename, const char *ca_dir) {
+int s2n_x509_trust_store_from_ca_file(struct s2n_x509_trust_store *store, const char *ca_pem_filename, const char *ca_dir)
+{
     if (!store->trust_store) {
         store->trust_store = X509_STORE_new();
         POSIX_ENSURE_REF(store->trust_store);
@@ -127,21 +130,22 @@ int s2n_x509_trust_store_from_ca_file(struct s2n_x509_trust_store *store, const
      * without a trust anchor. However if you call this function, the assumption is you trust ca_file or path and if a certificate
      * is encountered that's in that path, it should be trusted. The following flag tells libcrypto to not care that the cert
      * is missing a root anchor. */
-    unsigned long flags = X509_VP_FLAG_DEFAULT;
-    flags |=  X509_V_FLAG_PARTIAL_CHAIN;
+    unsigned long flags = X509_V_FLAG_PARTIAL_CHAIN;
     X509_STORE_set_flags(store->trust_store, flags);
 
     return 0;
 }
 
-void s2n_x509_trust_store_wipe(struct s2n_x509_trust_store *store) {
+void s2n_x509_trust_store_wipe(struct s2n_x509_trust_store *store)
+{
     if (store->trust_store) {
         X509_STORE_free(store->trust_store);
         store->trust_store = NULL;
     }
 }
 
-int s2n_x509_validator_init_no_x509_validation(struct s2n_x509_validator *validator) {
+int s2n_x509_validator_init_no_x509_validation(struct s2n_x509_validator *validator)
+{
     POSIX_ENSURE_REF(validator);
     validator->trust_store = NULL;
     validator->store_ctx = NULL;
@@ -155,7 +159,8 @@ int s2n_x509_validator_init_no_x509_validation(struct s2n_x509_validator *valida
     return 0;
 }
 
-int s2n_x509_validator_init(struct s2n_x509_validator *validator, struct s2n_x509_trust_store *trust_store, uint8_t check_ocsp) {
+int s2n_x509_validator_init(struct s2n_x509_validator *validator, struct s2n_x509_trust_store *trust_store, uint8_t check_ocsp)
+{
     POSIX_ENSURE_REF(trust_store);
     validator->trust_store = trust_store;
     validator->skip_cert_validation = 0;
@@ -173,13 +178,15 @@ int s2n_x509_validator_init(struct s2n_x509_validator *validator, struct s2n_x50
     return 0;
 }
 
-static inline void wipe_cert_chain(STACK_OF(X509) *cert_chain) {
+static inline void wipe_cert_chain(STACK_OF(X509) *cert_chain)
+{
     if (cert_chain) {
         sk_X509_pop_free(cert_chain, X509_free);
     }
 }
 
-int s2n_x509_validator_wipe(struct s2n_x509_validator *validator) {
+int s2n_x509_validator_wipe(struct s2n_x509_validator *validator)
+{
     if (validator->store_ctx) {
         X509_STORE_CTX_free(validator->store_ctx);
         validator->store_ctx = NULL;
@@ -198,7 +205,8 @@ int s2n_x509_validator_wipe(struct s2n_x509_validator *validator) {
     return S2N_SUCCESS;
 }
 
-int s2n_x509_validator_set_max_chain_depth(struct s2n_x509_validator *validator, uint16_t max_depth) {
+int s2n_x509_validator_set_max_chain_depth(struct s2n_x509_validator *validator, uint16_t max_depth)
+{
     POSIX_ENSURE_REF(validator);
     S2N_ERROR_IF(max_depth == 0, S2N_ERR_INVALID_ARGUMENT);
 
@@ -206,98 +214,168 @@ int s2n_x509_validator_set_max_chain_depth(struct s2n_x509_validator *validator,
     return 0;
 }
 
+static S2N_RESULT s2n_verify_host_information_san_entry(struct s2n_connection *conn, GENERAL_NAME *current_name, bool *san_found)
+{
+    RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_REF(current_name);
+    RESULT_ENSURE_REF(san_found);
+
+    if (current_name->type == GEN_DNS || current_name->type == GEN_URI) {
+        *san_found = true;
+
+        const char *name = (const char *) ASN1_STRING_data(current_name->d.ia5);
+        RESULT_ENSURE_REF(name);
+        int name_len = ASN1_STRING_length(current_name->d.ia5);
+        RESULT_ENSURE_GT(name_len, 0);
+
+        RESULT_ENSURE(conn->verify_host_fn(name, name_len, conn->data_for_verify_host), S2N_ERR_CERT_UNTRUSTED);
+
+        return S2N_RESULT_OK;
+    }
+
+    if (current_name->type == GEN_IPADD) {
+        *san_found = true;
+
+        /* try to validate an IP address if it's in the subject alt name. */
+        const unsigned char *ip_addr = current_name->d.iPAddress->data;
+        RESULT_ENSURE_REF(ip_addr);
+        int ip_addr_len = current_name->d.iPAddress->length;
+        RESULT_ENSURE_GT(ip_addr_len, 0);
+
+        RESULT_STACK_BLOB(address, INET6_ADDRSTRLEN + 1, INET6_ADDRSTRLEN + 1);
+
+        if (ip_addr_len == 4) {
+            RESULT_GUARD(s2n_inet_ntop(AF_INET, ip_addr, &address));
+        } else if (ip_addr_len == 16) {
+            RESULT_GUARD(s2n_inet_ntop(AF_INET6, ip_addr, &address));
+        } else {
+            /* we aren't able to parse this value so skip it */
+            RESULT_BAIL(S2N_ERR_CERT_UNTRUSTED);
+        }
+
+        /* strlen should be safe here since we made sure we were null terminated AND that inet_ntop succeeded */
+        const char *name = (const char *) address.data;
+        size_t name_len = strlen(name);
+
+        RESULT_ENSURE(conn->verify_host_fn(name, name_len, conn->data_for_verify_host), S2N_ERR_CERT_UNTRUSTED);
+
+        return S2N_RESULT_OK;
+    }
+
+    /* we don't understand this entry type so skip it */
+    RESULT_BAIL(S2N_ERR_CERT_UNTRUSTED);
+}
+
+static S2N_RESULT s2n_verify_host_information_san(struct s2n_connection *conn, X509 *public_cert, bool *san_found)
+{
+    RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_REF(public_cert);
+    RESULT_ENSURE_REF(san_found);
+
+    *san_found = false;
+
+    DEFER_CLEANUP(STACK_OF(GENERAL_NAME) *names_list = NULL, GENERAL_NAMES_free_pointer);
+    names_list = X509_get_ext_d2i(public_cert, NID_subject_alt_name, NULL, NULL);
+    RESULT_ENSURE(names_list, S2N_ERR_CERT_UNTRUSTED);
+
+    int n = sk_GENERAL_NAME_num(names_list);
+    RESULT_ENSURE(n > 0, S2N_ERR_CERT_UNTRUSTED);
+
+    s2n_result result = S2N_RESULT_OK;
+    for (int i = 0; i < n; i++) {
+        GENERAL_NAME *current_name = sk_GENERAL_NAME_value(names_list, i);
+
+        /* return success on the first entry that passes verification */
+        result = s2n_verify_host_information_san_entry(conn, current_name, san_found);
+        if (s2n_result_is_ok(result)) {
+            return S2N_RESULT_OK;
+        }
+    }
+
+    /* if an error was set by one of the entries, then just propagate the error from the last SAN entry call */
+    RESULT_GUARD(result);
+
+    RESULT_BAIL(S2N_ERR_CERT_UNTRUSTED);
+}
+
+static S2N_RESULT s2n_verify_host_information_common_name(struct s2n_connection *conn, X509 *public_cert, bool *cn_found)
+{
+    RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_REF(public_cert);
+    RESULT_ENSURE_REF(cn_found);
+
+    X509_NAME *subject_name = X509_get_subject_name(public_cert);
+    RESULT_ENSURE(subject_name, S2N_ERR_CERT_UNTRUSTED);
+
+    int curr_idx = -1;
+    while (true) {
+        int next_idx = X509_NAME_get_index_by_NID(subject_name, NID_commonName, curr_idx);
+        if (next_idx >= 0) {
+            curr_idx = next_idx;
+        } else {
+            break;
+        }
+    }
+
+    RESULT_ENSURE(curr_idx >= 0, S2N_ERR_CERT_UNTRUSTED);
+
+    ASN1_STRING *common_name = X509_NAME_ENTRY_get_data(X509_NAME_get_entry(subject_name, curr_idx));
+    RESULT_ENSURE(common_name, S2N_ERR_CERT_UNTRUSTED);
+
+    /* X520CommonName allows the following ANSI string types per RFC 5280 Appendix A.1 */
+    RESULT_ENSURE(ASN1_STRING_type(common_name) == V_ASN1_TELETEXSTRING
+                    || ASN1_STRING_type(common_name) == V_ASN1_PRINTABLESTRING
+                    || ASN1_STRING_type(common_name) == V_ASN1_UNIVERSALSTRING
+                    || ASN1_STRING_type(common_name) == V_ASN1_UTF8STRING
+                    || ASN1_STRING_type(common_name) == V_ASN1_BMPSTRING,
+            S2N_ERR_CERT_UNTRUSTED);
+
+    /* at this point we have a valid CN value */
+    *cn_found = true;
+
+    char peer_cn[255] = { 0 };
+    int len = ASN1_STRING_length(common_name);
+
+    RESULT_ENSURE_GT(len, 0);
+    RESULT_ENSURE_LTE(len, s2n_array_len(peer_cn) - 1);
+    RESULT_CHECKED_MEMCPY(peer_cn, ASN1_STRING_data(common_name), len);
+    RESULT_ENSURE(conn->verify_host_fn(peer_cn, len, conn->data_for_verify_host), S2N_ERR_CERT_UNTRUSTED);
+
+    return S2N_RESULT_OK;
+}
+
 /*
  * For each name in the cert. Iterate them. Call the callback. If one returns true, then consider it validated,
  * if none of them return true, the cert is considered invalid.
  */
-static uint8_t s2n_verify_host_information(struct s2n_x509_validator *validator, struct s2n_connection *conn, X509 *public_cert) {
-    (void)validator;
-    uint8_t verified = 0;
-    uint8_t san_found = 0;
+static S2N_RESULT s2n_verify_host_information(struct s2n_connection *conn, X509 *public_cert)
+{
+    bool entry_found = false;
 
     /* Check SubjectAltNames before CommonName as per RFC 6125 6.4.4 */
-    STACK_OF(GENERAL_NAME) *names_list = X509_get_ext_d2i(public_cert, NID_subject_alt_name, NULL, NULL);
-    int n = sk_GENERAL_NAME_num(names_list);
-    for (int i = 0; i < n && !verified; i++) {
-        GENERAL_NAME *current_name = sk_GENERAL_NAME_value(names_list, i);
-        if (current_name->type == GEN_DNS) {
-            san_found = 1;
-
-            const char *name = (const char *) ASN1_STRING_data(current_name->d.ia5);
-            size_t name_len = (size_t) ASN1_STRING_length(current_name->d.ia5);
-
-            verified = conn->verify_host_fn(name, name_len, conn->data_for_verify_host);
-        } else if (current_name->type == GEN_URI) {
-            const char *name = (const char *) ASN1_STRING_data(current_name->d.ia5);
-            size_t name_len = (size_t) ASN1_STRING_length(current_name->d.ia5);
-
-            verified = conn->verify_host_fn(name, name_len, conn->data_for_verify_host);
-        } else if (current_name->type == GEN_IPADD) {
-            san_found = 1;
-            /* try to validate an IP address if it's in the subject alt name. */
-            const unsigned char *ip_addr = current_name->d.iPAddress->data;
-            size_t ip_addr_len = (size_t)current_name->d.iPAddress->length;
-
-            s2n_result parse_result = S2N_RESULT_ERROR;
-            s2n_stack_blob(address, INET6_ADDRSTRLEN + 1, INET6_ADDRSTRLEN + 1);
-            if (ip_addr_len == 4) {
-                parse_result = s2n_inet_ntop(AF_INET, ip_addr, &address);
-            } else if (ip_addr_len == 16) {
-                parse_result = s2n_inet_ntop(AF_INET6, ip_addr, &address);
-            }
-
-            /* strlen should be safe here since we made sure we were null terminated AND that inet_ntop succeeded */
-            if (s2n_result_is_ok(parse_result)) {
-                verified = conn->verify_host_fn(
-                               (const char *)address.data,
-                               strlen((const char *)address.data),
-                               conn->data_for_verify_host);
-            }
-        }
+    s2n_result result = s2n_verify_host_information_san(conn, public_cert, &entry_found);
+    if (entry_found) {
+        return result;
     }
 
-    GENERAL_NAMES_free(names_list);
-
     /* if no SubjectAltNames of type DNS found, go to the common name. */
-    if (!verified && !san_found) {
-        X509_NAME *subject_name = X509_get_subject_name(public_cert);
-        if (subject_name) {
-            int next_idx = 0, curr_idx = -1;
-            while ((next_idx = X509_NAME_get_index_by_NID(subject_name, NID_commonName, curr_idx)) >= 0) {
-                curr_idx = next_idx;
-            }
-
-            if (curr_idx >= 0) {
-                ASN1_STRING *common_name =
-                        X509_NAME_ENTRY_get_data(X509_NAME_get_entry(subject_name, curr_idx));
-
-                if (common_name) {
-                    char peer_cn[255];
-                    static size_t peer_cn_size = sizeof(peer_cn);
-                    POSIX_CHECKED_MEMSET(&peer_cn, 0, peer_cn_size);
-
-                    /* X520CommonName allows the following ANSI string types per RFC 5280 Appendix A.1 */
-                    if (ASN1_STRING_type(common_name) == V_ASN1_TELETEXSTRING ||
-                        ASN1_STRING_type(common_name) == V_ASN1_PRINTABLESTRING ||
-                        ASN1_STRING_type(common_name) == V_ASN1_UNIVERSALSTRING ||
-                        ASN1_STRING_type(common_name) == V_ASN1_UTF8STRING ||
-                        ASN1_STRING_type(common_name) == V_ASN1_BMPSTRING ) {
-
-                        size_t len = (size_t) ASN1_STRING_length(common_name);
-
-                        POSIX_ENSURE_LTE(len, sizeof(peer_cn) - 1);
-                        POSIX_CHECKED_MEMCPY(peer_cn, ASN1_STRING_data(common_name), len);
-                        verified = conn->verify_host_fn(peer_cn, len, conn->data_for_verify_host);
-                    }
-                }
-            }
-        }
+    result = s2n_verify_host_information_common_name(conn, public_cert, &entry_found);
+    if (entry_found) {
+        return result;
     }
 
-    return verified;
+    /* make a null-terminated string in case the callback tries to use strlen */
+    const char *name = "";
+    size_t name_len = 0;
+
+    /* at this point, we don't have anything to identify the certificate with so pass an empty string to the callback */
+    RESULT_ENSURE(conn->verify_host_fn(name, name_len, conn->data_for_verify_host), S2N_ERR_CERT_UNTRUSTED);
+
+    return S2N_RESULT_OK;
 }
 
-static S2N_RESULT s2n_x509_validator_read_asn1_cert(struct s2n_stuffer *cert_chain_in_stuffer, struct s2n_blob *asn1_cert) {
+static S2N_RESULT s2n_x509_validator_read_asn1_cert(struct s2n_stuffer *cert_chain_in_stuffer, struct s2n_blob *asn1_cert)
+{
     uint32_t certificate_size = 0;
 
     RESULT_GUARD_POSIX(s2n_stuffer_read_uint24(cert_chain_in_stuffer, &certificate_size));
@@ -312,11 +390,13 @@ static S2N_RESULT s2n_x509_validator_read_asn1_cert(struct s2n_stuffer *cert_cha
 }
 
 static S2N_RESULT s2n_x509_validator_read_cert_chain(struct s2n_x509_validator *validator, struct s2n_connection *conn,
-        uint8_t *cert_chain_in, uint32_t cert_chain_len) {
+        uint8_t *cert_chain_in, uint32_t cert_chain_len)
+{
     RESULT_ENSURE(validator->skip_cert_validation || s2n_x509_trust_store_has_certs(validator->trust_store), S2N_ERR_CERT_UNTRUSTED);
     RESULT_ENSURE(validator->state == INIT, S2N_ERR_INVALID_CERT_STATE);
 
-    struct s2n_blob cert_chain_blob = {.data = cert_chain_in, .size = cert_chain_len};
+    struct s2n_blob cert_chain_blob = { 0 };
+    RESULT_GUARD_POSIX(s2n_blob_init(&cert_chain_blob, cert_chain_in, cert_chain_len));
     DEFER_CLEANUP(struct s2n_stuffer cert_chain_in_stuffer = { 0 }, s2n_stuffer_free);
 
     RESULT_GUARD_POSIX(s2n_stuffer_init(&cert_chain_in_stuffer, &cert_chain_blob));
@@ -324,7 +404,8 @@ static S2N_RESULT s2n_x509_validator_read_cert_chain(struct s2n_x509_validator *
 
     X509 *server_cert = NULL;
 
-    while (s2n_stuffer_data_available(&cert_chain_in_stuffer) && sk_X509_num(validator->cert_chain_from_wire) < validator->max_chain_depth) {
+    while (s2n_stuffer_data_available(&cert_chain_in_stuffer)
+            && sk_X509_num(validator->cert_chain_from_wire) < validator->max_chain_depth) {
         struct s2n_blob asn1_cert = { 0 };
         RESULT_GUARD(s2n_x509_validator_read_asn1_cert(&cert_chain_in_stuffer, &asn1_cert));
 
@@ -362,7 +443,8 @@ static S2N_RESULT s2n_x509_validator_read_cert_chain(struct s2n_x509_validator *
 }
 
 static S2N_RESULT s2n_x509_validator_process_cert_chain(struct s2n_x509_validator *validator, struct s2n_connection *conn,
-        uint8_t *cert_chain_in, uint32_t cert_chain_len) {
+        uint8_t *cert_chain_in, uint32_t cert_chain_len)
+{
     RESULT_ENSURE(validator->state == INIT, S2N_ERR_INVALID_CERT_STATE);
 
     RESULT_GUARD(s2n_x509_validator_read_cert_chain(validator, conn, cert_chain_in, cert_chain_len));
@@ -375,11 +457,12 @@ static S2N_RESULT s2n_x509_validator_process_cert_chain(struct s2n_x509_validato
     RESULT_ENSURE_REF(leaf);
 
     if (conn->verify_host_fn) {
-        RESULT_ENSURE(s2n_verify_host_information(validator, conn, leaf), S2N_ERR_CERT_UNTRUSTED);
+        RESULT_GUARD(s2n_verify_host_information(conn, leaf));
     }
 
     RESULT_GUARD_OSSL(X509_STORE_CTX_init(validator->store_ctx, validator->trust_store->trust_store, leaf,
-            validator->cert_chain_from_wire), S2N_ERR_INTERNAL_LIBCRYPTO_ERROR);
+                              validator->cert_chain_from_wire),
+            S2N_ERR_INTERNAL_LIBCRYPTO_ERROR);
 
     if (conn->config->crl_lookup_cb) {
         RESULT_GUARD(s2n_crl_invoke_lookup_callbacks(conn, validator));
@@ -391,7 +474,8 @@ static S2N_RESULT s2n_x509_validator_process_cert_chain(struct s2n_x509_validato
     return S2N_RESULT_OK;
 }
 
-static S2N_RESULT s2n_x509_validator_verify_cert_chain(struct s2n_x509_validator *validator, struct s2n_connection *conn) {
+static S2N_RESULT s2n_x509_validator_verify_cert_chain(struct s2n_x509_validator *validator, struct s2n_connection *conn)
+{
     RESULT_ENSURE(validator->state == READY_TO_VERIFY, S2N_ERR_INVALID_CERT_STATE);
 
     X509_VERIFY_PARAM *param = X509_STORE_CTX_get0_param(validator->store_ctx);
@@ -421,7 +505,7 @@ static S2N_RESULT s2n_x509_validator_verify_cert_chain(struct s2n_x509_validator
     RESULT_GUARD(s2n_config_wall_clock(conn->config, &current_sys_time));
 
     /* this wants seconds not nanoseconds */
-    time_t current_time = (time_t)(current_sys_time / 1000000000);
+    time_t current_time = (time_t) (current_sys_time / 1000000000);
     X509_STORE_CTX_set_time(validator->store_ctx, 0, current_time);
 
     int verify_ret = X509_verify_cert(validator->store_ctx);
@@ -452,8 +536,10 @@ static S2N_RESULT s2n_x509_validator_verify_cert_chain(struct s2n_x509_validator
 }
 
 static S2N_RESULT s2n_x509_validator_read_leaf_info(struct s2n_connection *conn, uint8_t *cert_chain_in, uint32_t cert_chain_len,
-        struct s2n_pkey *public_key, s2n_pkey_type *pkey_type, s2n_parsed_extensions_list *first_certificate_extensions) {
-    struct s2n_blob cert_chain_blob = {.data = cert_chain_in, .size = cert_chain_len};
+        struct s2n_pkey *public_key, s2n_pkey_type *pkey_type, s2n_parsed_extensions_list *first_certificate_extensions)
+{
+    struct s2n_blob cert_chain_blob = { 0 };
+    RESULT_GUARD_POSIX(s2n_blob_init(&cert_chain_blob, cert_chain_in, cert_chain_len));
     DEFER_CLEANUP(struct s2n_stuffer cert_chain_in_stuffer = { 0 }, s2n_stuffer_free);
 
     RESULT_GUARD_POSIX(s2n_stuffer_init(&cert_chain_in_stuffer, &cert_chain_blob));
@@ -477,7 +563,8 @@ static S2N_RESULT s2n_x509_validator_read_leaf_info(struct s2n_connection *conn,
 }
 
 S2N_RESULT s2n_x509_validator_validate_cert_chain(struct s2n_x509_validator *validator, struct s2n_connection *conn,
-        uint8_t *cert_chain_in, uint32_t cert_chain_len, s2n_pkey_type *pkey_type, struct s2n_pkey *public_key_out) {
+        uint8_t *cert_chain_in, uint32_t cert_chain_len, s2n_pkey_type *pkey_type, struct s2n_pkey *public_key_out)
+{
     switch (validator->state) {
         case INIT:
             break;
@@ -522,8 +609,8 @@ S2N_RESULT s2n_x509_validator_validate_cert_chain(struct s2n_x509_validator *val
 }
 
 S2N_RESULT s2n_x509_validator_validate_cert_stapled_ocsp_response(struct s2n_x509_validator *validator,
-        struct s2n_connection *conn, const uint8_t *ocsp_response_raw, uint32_t ocsp_response_length) {
-
+        struct s2n_connection *conn, const uint8_t *ocsp_response_raw, uint32_t ocsp_response_length)
+{
     if (validator->skip_cert_validation || !validator->check_stapled_ocsp) {
         validator->state = OCSP_VALIDATED;
         return S2N_RESULT_OK;
@@ -539,7 +626,7 @@ S2N_RESULT s2n_x509_validator_validate_cert_stapled_ocsp_response(struct s2n_x50
     RESULT_ENSURE_REF(ocsp_response_raw);
 
     DEFER_CLEANUP(OCSP_RESPONSE *ocsp_response = d2i_OCSP_RESPONSE(NULL, &ocsp_response_raw, ocsp_response_length),
-                  OCSP_RESPONSE_free_pointer);
+            OCSP_RESPONSE_free_pointer);
     RESULT_ENSURE(ocsp_response != NULL, S2N_ERR_INVALID_OCSP_RESPONSE);
 
     int ocsp_status = OCSP_response_status(ocsp_response);
@@ -650,7 +737,8 @@ S2N_RESULT s2n_validate_certificate_signature(struct s2n_connection *conn, X509
     return S2N_RESULT_OK;
 }
 
-S2N_RESULT s2n_validate_sig_scheme_supported(struct s2n_connection *conn, X509 *x509_cert, const struct s2n_signature_preferences *cert_sig_preferences)
+S2N_RESULT s2n_validate_sig_scheme_supported(struct s2n_connection *conn, X509 *x509_cert,
+        const struct s2n_signature_preferences *cert_sig_preferences)
 {
     RESULT_ENSURE_REF(conn);
     RESULT_ENSURE_REF(x509_cert);
@@ -658,19 +746,19 @@ S2N_RESULT s2n_validate_sig_scheme_supported(struct s2n_connection *conn, X509 *
 
     int nid = 0;
 
-    #if defined(LIBRESSL_VERSION_NUMBER) && (LIBRESSL_VERSION_NUMBER < 0x02070000f)
-        RESULT_ENSURE_REF(x509_cert->sig_alg);
-        nid = OBJ_obj2nid(x509_cert->sig_alg->algorithm);
-    #else
-        nid = X509_get_signature_nid(x509_cert);
-    #endif
+#if defined(LIBRESSL_VERSION_NUMBER) && (LIBRESSL_VERSION_NUMBER < 0x02070000f)
+    RESULT_ENSURE_REF(x509_cert->sig_alg);
+    nid = OBJ_obj2nid(x509_cert->sig_alg->algorithm);
+#else
+    nid = X509_get_signature_nid(x509_cert);
+#endif
 
     for (size_t i = 0; i < cert_sig_preferences->count; i++) {
-
         if (cert_sig_preferences->signature_schemes[i]->libcrypto_nid == nid) {
             /* SHA-1 algorithms are not supported in certificate signatures in TLS1.3 */
-            RESULT_ENSURE(!(conn->actual_protocol_version >= S2N_TLS13 &&
-                    cert_sig_preferences->signature_schemes[i]->hash_alg == S2N_HASH_SHA1), S2N_ERR_CERT_UNTRUSTED);
+            RESULT_ENSURE(!(conn->actual_protocol_version >= S2N_TLS13
+                                  && cert_sig_preferences->signature_schemes[i]->hash_alg == S2N_HASH_SHA1),
+                    S2N_ERR_CERT_UNTRUSTED);
 
             return S2N_RESULT_OK;
         }
